import { Injectable } from '@angular/core';
import * as Mqtt from 'mqtt'

type callbackFunc = (message: string) => void

@Injectable()
export class MqttclientService {
  private mqttClient : Mqtt.MqttClient
  private callbacks : Map<string, callbackFunc> = new Map()

  constructor() { 
    this.mqttClient = Mqtt.connect('ws://localhost:9001')
    this.mqttClient.on('connect', () => {
      console.log('MQTT connected')
      this.callbacks.forEach((value: callbackFunc, key: string) => {
        this.mqttClient.subscribe(key)
        console.log(`${key} subscribed`)
      })
    })
    this.mqttClient.on('message', (topic: string, messageBuf: Buffer) => {
      if (this.callbacks.has(topic)) {
        let message : string  = messageBuf.toString('UTF-8')
        console.log(`Providing ${message} to callback for ${topic}`)
        this.callbacks.get(topic)(message)
      }
    })
  }

  register(topic: string, cb: callbackFunc) {
    this.callbacks.set(topic, cb)
    this.mqttClient.subscribe(topic)
  }

  publish(topic: string, message: string) {
    console.log(`${message} published on ${topic}`)
    this.mqttClient.publish(topic, message)
  }
}
